package com.cmeeting;
import com.azure.core.credential.TokenRequestContext;
import com.azure.identity.ClientSecretCredential;
import com.azure.identity.ClientSecretCredentialBuilder;
import com.microsoft.graph.authentication.TokenCredentialAuthProvider;
import com.microsoft.graph.http.GraphServiceException;
import com.microsoft.graph.models.Message;
import com.microsoft.graph.requests.GraphServiceClient;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import com.microsoft.graph.models.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.Collections;

public class SendMailExample {
    private static final Logger logger = LoggerFactory.getLogger(OutlookTest.class);

    public static void main(String[] args) {
        // 1. Azure AD 配置
        final String clientId = "d65aa91b-05f4-42b2-902f-4d82ea5a2b93";
        final String tenantId = "74cc8acf-aacc-4514-9bbb-dc27ce3096bb";
        final String clientSecret = "~N98Q~83v6dViQFIofwr0fRn4J5VEZ2tvwOz.bPX";


        // 2. 创建认证凭据
        ClientSecretCredential credential = new ClientSecretCredentialBuilder()
                .clientId(clientId)
                .tenantId(tenantId)
                .clientSecret(clientSecret)
                .build();
        String token = credential.getToken(
                        new TokenRequestContext()
                                .addScopes("https://graph.microsoft.com/.default"))
                .block().getToken();
        System.out.println("获取到的令牌:"+ token);

        // 3. 初始化认证提供者
        TokenCredentialAuthProvider authProvider = new TokenCredentialAuthProvider(
                Collections.singletonList("https://graph.microsoft.com/.default"),
                credential);
        // 4. 创建 Graph 客户端
        GraphServiceClient<Request> graphClient = GraphServiceClient.builder()
                .authenticationProvider(authProvider)
                .httpClient(createHttpClient())
                .buildClient();

        // 5. 构建邮件内容
        Message message = new Message();
        message.subject = "Test from Java 8";

        ItemBody body = new ItemBody();
        body.contentType = BodyType.TEXT;
        body.content = "This works with Java 8!";
        message.body = body;

        EmailAddress emailAddress = new EmailAddress();
        emailAddress.address = "binzhai321@outlook.com";

        Recipient recipient = new Recipient();
        recipient.emailAddress = emailAddress;
        message.toRecipients = Collections.singletonList(recipient);
        // 最兼容的发送方式
        try {
            // 创建参数集
            UserSendMailParameterSet parameterSet = UserSendMailParameterSet.newBuilder()
                    .withMessage(message)
                    .withSaveToSentItems(true)
                    .build();
            // 指定目标用户的 ID 或邮箱地址
            String userId = "binzhai321@outlook.com";
            // 发送邮件
            graphClient.users(userId)
                    .sendMail(parameterSet)
                    .buildRequest()
                    .post();

            System.out.println("Email sent successfully!");
        } catch (GraphServiceException e) {
            System.err.println("Error code: " + e.getServiceError().code);
            System.err.println("Error message: " + e.getServiceError().message);
            System.err.println("Headers: " + e.getRequestHeaders());
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
    }
    private static OkHttpClient createHttpClient() {
        return new OkHttpClient.Builder()
                .addInterceptor(chain -> {
                    okhttp3.Request request = chain.request();

                    // 打印请求信息
                    System.out.println("--> " + request.method() + " " + request.url());
                    if (request.body() != null) {
                        System.out.println("Request Body: " + request.body().toString());
                    }
                    if (request.headers() != null) {
                        System.out.println("Request Headers: " + request.headers().toString());
                    }

                    okhttp3.Response response = chain.proceed(request);

                    // 打印响应信息
                    if (response.body() != null) {
                        String responseBody = response.body().string();
                        System.out.println("<-- " + response.code() + " " + response.message() + " " + response.request().url());
                        System.out.println("Response Body: " + responseBody);
                        // 重新包装响应体
                        response = response.newBuilder()
                                .body(okhttp3.ResponseBody.create(responseBody, okhttp3.MediaType.parse("application/json")))
                                .build();
                    }
                    return response;
                })
                .build();
    }
}